/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Computes the boundary of the graphs. */
/*                                      */

package OutH;

import java.util.Vector;
import java.io.*;

public class Boundary
{
  public static void main(String argv[]) throws IOException {
    int gix, i, j, k, p, r[];
    Graph g = null, g1 = null;
    Tree t, t1;
    Vector column, graphBag;
    int[] mxelt = new int[3], oldelt = new int[3];
    boolean found;
    Permutation sigma;
    
    if (argv.length < 3) {
      System.out.println("Usage: Boundary <original graphs> "+
                         "<contracted graphs> <boundary matrix>.");
      System.exit(1);
      }

    System.out.print("Reading graphs ...");
    FileInputStream infs = new FileInputStream(argv[0]);             
    ObjectInputStream ins = new ObjectInputStream(infs);
    FileInputStream outfs = new FileInputStream(argv[1]);
    ObjectInputStream outs = new ObjectInputStream(outfs);
    FileOutputStream matrixfs = new FileOutputStream(argv[2], false);
    PrintWriter matrixs = new PrintWriter(matrixfs);
    
    graphBag = new Vector(100,100);
    try {
        while (true) {
          g = (Graph)outs.readObject();
          graphBag.addElement(g);
          }
        }
      catch(EOFException e) { }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
    outfs.close();
    
    column = new Vector(100,100);
    
    System.out.print(" starting ...");
    i = -1;
    while(true) { 
      try {
        g = (Graph)ins.readObject();
        }
      catch (EOFException e) {
        break;
        }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
      i++;
      System.out.print(" "+ i +"("+ g.treeBag.size() +")");
      matrixs.println();
      matrixs.println("Graph "+i);
      
      for (j=0; j<g.treeBag.size(); j++) {
        column.removeAllElements();
        t = (Tree)g.treeBag.elementAt(j);

// boundary plus

        gix = findGraph(graphBag, g);
        
        for (k=0; k<t.allEdges; k++) {
          t1 = new Tree(t);
          t1.removeEdge(k);
          if (t1.hasOddSymmetry()) {
            continue;
            }
          t1.toNormalForm();
          mxelt = findTree(graphBag, gix, t1);	// find index of t1
          if (k%2 == 0)
            mxelt[0] = -mxelt[0];		// multiply coeff by (-1)^k+1
          found = false;
          for (p=0; p<column.size(); p++) {
            oldelt = (int[])column.elementAt(p);
            if (gix == oldelt[1] && mxelt[2] == oldelt[2]) {
              if ((oldelt[0] += mxelt[0]) == 0)	// increase coefficient
                column.removeElementAt(p);		// if tree occurred in
              found = true;						// this column already...
              break;
              }
            }
          if (!found) {							// ... add new element
            mxelt[1] = gix;						// to column if not.
            column.addElement(mxelt.clone());	
            }
          }
          
// boundary minus

        for (k=0; k<t.allEdges; k++) {
          g1 = new Graph(g);
          t1 = new Tree(t);
          t1.parent = g1;
          t1.contractEdge(k);		// contract the tree and its parent
          sigma = g1.toNormalForm();
          t1.permute(sigma);
          if (t1.hasOddSymmetry()) {
            continue;
            }
          t1.toNormalForm();       
          mxelt = findTree(graphBag, t1);	// find index of t1
          if (k%2 != 0)
            mxelt[0] = -mxelt[0];			// multiply coeff by (-1)^k
          found = false;
          for (p=0; p<column.size(); p++) {
            oldelt = (int[])column.elementAt(p);
            if (mxelt[1] == oldelt[1] && mxelt[2] == oldelt[2]) {
              if ((oldelt[0] += mxelt[0]) == 0)		// increase coefficient
                column.removeElementAt(p);			// if tree occurred in
              found = true;							// this column already...
              break;
              }
            }
          if (!found)								// ... add new element
            column.addElement(mxelt.clone());		// to column if not
          }
          
        for (k=0; k<column.size(); k++) {
          mxelt = (int[])column.elementAt(k);
          matrixs.print(mxelt[0]+" "+mxelt[1]+" "+mxelt[2]+"  ");
          }
        matrixs.println(";");
        }	// for(j) loop; process next subtree
      }	  // while(true) loop; read next graph
            
    ins.close();   infs.close();
    matrixs.flush();  matrixs.close();  matrixfs.close();

    System.out.println("done. "+ new java.util.Date());
    }


  private static int findGraph(Vector bag, Graph graph) {
    int i;
   
    for (i=0; i<bag.size(); i++)
      if (graph.equals((Graph)bag.elementAt(i)))
        return i;					
    System.out.println("Error: can't find graph.");
    return -1;
    }


  private static int[] findTree(Vector bag, int index, Tree tree) {
    int i, r[] = new int[3];
    
    Graph graph = (Graph)bag.elementAt(index);
    for (i=0; i<graph.treeBag.size(); i++)
      if (tree.equals((Tree)graph.treeBag.elementAt(i))) {
        r[0] = tree.coeff;
        r[1] = index;
        r[2] = i;
        return r;
        }
    System.out.println("Error: can't find tree.");
    return null;    
    }


  private static int[] findTree(Vector bag, Tree tree) {
    int index, r[] = new int[3];
    
    index = findGraph(bag, tree.parent);
    return findTree(bag, index, tree);
    }

}
