/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Graph class: a SimpleGraph           */
/* plus a list of subtrees and a list of automorphisms */

package OutH;

import java.util.Vector;

public class Graph extends SimpleGraph implements java.io.Serializable
{
  public Vector treeBag;	// subtrees of the graph
  public Vector symBag;		// the automorphisms of the graph
  
  public Graph(int n) {				// a new graph of size n
    super(n);
    treeBag = new Vector(100,100);
    symBag = new Vector(10,10);
    }
   
   
  public Graph(Graph g) {			// a new copy of the graph g
    super(g);
    treeBag = new Vector(100,100);
    symBag = new Vector(10,10);
    }
    

  public void permute(Permutation sigma) {
    int i;
    Permutation tau;
    SimpleGraph t;			// change to Tree!!!
    
    super.permute(sigma);
    for (i=0; i<symBag.size(); i++) {
      tau = (Permutation)symBag.elementAt(i);
      tau.conjugate(sigma);
      }
    for (i=0; i<treeBag.size(); i++) {
      t = (Tree)treeBag.elementAt(i);
      t.permute(sigma);
      }
    }
    

  public Permutation normalize() {
    int i, j, n, nn, buf[];
    Permutation s;
        
    noEdges = 0;					// find correct value of noEdges,
    buf = new int[size];			//  noLoops and noMults; 
    for (i=0; i<size; i++) 			//  and move special vertices
      if (m[i][i] != 0)				//  to the front
        buf[i] = 1;
    
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++) {
        if (m[i][j] != 0)
          noEdges++;
        if (m[i][j] > 1) {
          if (buf[i] == 0)
            buf[i] = 2;
          if (buf[j] == 0)
            buf[j] = 2;
          }
        }
      
    s = new Permutation(size);
    n = 0;
    for (i=0; i<size; i++)
      if (buf[i] == 1)
        s.p[i] = n++;
    noLoops = nn = n;
    
    for (i=0; i<size; i++)
      if (buf[i] == 2)
        s.p[i] = n++;
    noMults = n - nn;
    
    for (i=0; i<size; i++)
      if (buf[i] == 0)
        s.p[i] = n++;
        
    permute(s);
    return s;
    }
    

  public Permutation toNormalForm() {
    symBag.removeAllElements();		// probably unnecessary
    Permutation sigma = normalize();
    sigma.rightMultiply(super.toNormalForm(this.symBag));
    return sigma;
    }      
      

  public Graph addEdge(int edge1, int edge2) {
    int i, j, r[];
    Graph g = new Graph(size + 2);	// add an extra edge between
									// the midpoints of the edge1th
    for (i=0; i<size; i++)			// and the edge2th edges
      for (j=i; j<size; j++)
        g.m[i][j] = m[i][j];
        
    if (edge1 != edge2) {    
      r = this.kthEdge(edge1);			// we are assuming there are
      g.m[r[0]][r[1]]--;				// no loops, but there may be
      g.m[r[0]][size] = g.m[r[1]][size] = 1;	// multiple edges
         
      r = this.kthEdge(edge2);
      g.m[r[0]][r[1]]--;
      g.m[r[0]][size+1] = g.m[r[1]][size+1] = 1;
    
      g.m[size][size+1] = 1;
      }
    else {
      r = this.kthEdge(edge1);
      g.m[r[0]][r[1]]--;
      g.m[r[0]][size] = g.m[r[1]][size+1] = 1;    
      g.m[size][size+1] = 2;
      }
      
    g.allEdges = allEdges + 3;	// -2 +5 or -1 +4  net change
    // g.noEdges, g.noLoops and g.noMults need to be fixed later

    return g;
    }


  public String toString() {
    String str = super.toString() +"; trees: "+ treeBag.size() +
                  "  syms: "+ symBag.size() +"\r\n";   
    return str;
    }  

}
