/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Computes the kernel of a matrix by simple Gaussian elimination. */

package OutH;

import java.util.Vector;
import java.io.*;

public class Kernel
{
  final static public int MAXSIZE = 50000;

  public static void main(String argv[]) throws IOException 
  {
    int i, j, k, p, mxsize, kerneldim, imagedim, ans = 0;
    Vector column, matrix[], mirror[];
    long[][] columnid;
    long[] mxelt = new long[3], oldelt = new long[3];
    long mult, subt, q, largestCoeff;
    boolean found;
    FileInputStream infs;
    StreamTokenizer ins;
    FileOutputStream kerfs, imgfs;      
    PrintWriter kers, imgs;    
          
    if (argv.length < 3) {
      System.out.println("Usage: Kernel <input> <kernel> <image>.");
      System.exit(1);
      }

    infs = new FileInputStream(argv[0]);             
    ins = new StreamTokenizer(new InputStreamReader(infs));
    kerfs = new FileOutputStream(argv[1], false);             
    kers = new PrintWriter(kerfs);
    imgfs = new FileOutputStream(argv[2], false);             
    imgs = new PrintWriter(imgfs);

    ins.lowerCaseMode(false);
    ins.eolIsSignificant(false);
    ins.parseNumbers();
    
    matrix = new Vector[MAXSIZE];
    columnid = new long[MAXSIZE][2];
    mxsize = 0;
    i = -1;
    j = 0;
    column = new Vector(100,100);
    
    System.out.print("Reading matrix...  ");
    while (true) {
      column.removeAllElements();
      ans  = readLine(ins, column);
      if (ans == COLUMN) {
        if (mxsize >= MAXSIZE) {
          System.out.println("Hey! This matrix has more than " + 
                           MAXSIZE +" columns.  Give me a break...");
          System.exit(1);
          } 
        matrix[mxsize] = (Vector) column.clone();
        columnid[mxsize][0] = i;
        columnid[mxsize][1] = j++;
        mxsize++;
        }
      else if (ans == EOF)
        break;
      else {	// found a "Graph x" line
        i = ans; 
        j = 0;
        }
      }
    System.out.println("Read "+ mxsize +" columns.");

    mirror = new Vector[mxsize];
    for (j=0; j<mxsize; j++) {
      mirror[j] = new Vector(100, 100);
      mxelt = new long[3];						// the identity matrix
      mxelt[0] = 1;  
      mxelt[1] = columnid[j][0];
      mxelt[2] = columnid[j][1];      
      mirror[j].addElement(mxelt);
      } 

    System.out.print("Eliminating... ");
    kerneldim = imagedim = 0;         
    for (j=0; j<mxsize; j++) {        		// compute the kernel

        if (matrix[j].size() == 0) {		// zero column in matrix:
          if (mirror[j].size() > 0) {		// found a kernel element
            LinAlgUtil.reduce(mirror[j]);
            for (k=0; k<mirror[j].size(); k++) {
              mxelt = (long[])mirror[j].elementAt(k);
              kers.print(mxelt[0]+" "+ mxelt[1] +" "+mxelt[2]+"  ");
              }
            }
          kers.println(";");
          kerneldim++;
          System.out.print("*");
          matrix[j] = mirror[j] = null;
          continue;
          }
       			// non-zero column in matrix: a basis vector of the image
        for (k=0; k<matrix[j].size(); k++) {
          mxelt = (long[])matrix[j].elementAt(k);
          imgs.print(mxelt[0] +" "+ mxelt[1] +" "+mxelt[2]+"  ");
          }
        imgs.println(";");
        imagedim++;
       
        coReduce(matrix[j], mirror[j]);		// divide by common factor, if any
            
        mxelt = (long[])matrix[j].elementAt(0); 
        System.out.print("("+ j +","+ mxelt[0]);
        largestCoeff = 0;

        for (k=j+1; k<mxsize; k++) {				// check each subsequent
          boolean occurs = false;					// column if the main 
          for (p=0; p<matrix[k].size(); p++) {		// coeff of matrix[j]
            oldelt = (long[])(matrix[k].elementAt(p));	// occurs in it
            if (mxelt[1] == oldelt[1] && mxelt[2] == oldelt[2]) {
              occurs = true; 
              break;
              }
           }
         if (occurs) {
            mult = mxelt[0];
            subt = oldelt[0];
            q = LinAlgUtil.gcd(mult, subt);
            mult = mult/q;
            subt = subt/q;
            if (subt > largestCoeff)
              largestCoeff = subt;
            if (subt < -largestCoeff)						// this it the
              largestCoeff = -subt;							// Gaussian elim.
            LinAlgUtil.multiply(matrix[k], mult);			// part gufe
            LinAlgUtil.addColumn(matrix[j], -subt, matrix[k]);
            LinAlgUtil.multiply(mirror[k], mult);
            LinAlgUtil.addColumn(mirror[j], -subt, mirror[k]);
            coReduce(matrix[k],mirror[k]);
            }
          }
        matrix[j] = mirror[j] = null;
        System.out.print(","+ largestCoeff +")");
        }
     
    infs.close();
    kers.close();  kerfs.close();
    imgs.close();  imgfs.close();

    System.out.println();
    System.out.println("kernel dim = "+ kerneldim +"; image dim = "+
          imagedim +"  "+ new java.util.Date());
    }
    
    
  public static void coReduce(Vector one, Vector two) {		// divide both
    int i;													// colums by their
    long q;													// common factor
    
    q = LinAlgUtil.gcd(one);
    q = LinAlgUtil.gcd(q, LinAlgUtil.gcd(two));
    if (q > 1) {
      LinAlgUtil.divide(one, q);
      LinAlgUtil.divide(two, q);
      }
    }
    
   
  final static public int COLUMN = -2;
  final static public int EOF = -1;	// non-neg values mean "Graph X" lines
  
  public static int readLine(StreamTokenizer s, Vector bag)
                                                      throws IOException {
    long mxelt[] = new long[3];
       
    s.nextToken();
    if (s.ttype == StreamTokenizer.TT_EOF)
      return EOF;
      
    while (s.ttype == StreamTokenizer.TT_NUMBER) {
      mxelt[0] = (long)s.nval;
      s.nextToken();
      mxelt[1] = (long)s.nval;
      s.nextToken();
      mxelt[2] = (long)s.nval;
      s.nextToken();
      bag.addElement(mxelt.clone());
      }
    
    if (s.ttype == (int)';')
      return COLUMN;
    
    if (s.ttype == StreamTokenizer.TT_WORD && s.sval.equals("Graph")) {
      s.nextToken();
      return (int)s.nval;
      }
    
    System.out.println("Error reading "+s);
    System.exit(1);
    return EOF;		// well, it doesn't, but the compiler requires it
    }      
    
}
