/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Computes the kernel of a matrix by simple Gaussian elimination.    */
/*                                                                    */
/* This is the same as OutH/Kernel.java, except it does not write     */
/* the kernel and boundary matrices to disk.  As long as the homology */
/* is zero, these are not necessary, anyway.                          */

package OutH;

import java.util.Vector;
import java.io.*;

public class KernelCount
{
  final static public int MAXSIZE = 50000;

  public static void main(String argv[]) throws IOException 
  {
    int i, j, k, p, mxsize, kerneldim, imagedim, ans = 0;
    Vector column, matrix[];
    long[][] columnid;
    long[] mxelt = new long[3], oldelt = new long[3];
    long mult, subt, q, largestCoeff;
    boolean found;
    FileInputStream infs;
    StreamTokenizer ins;
          
    if (argv.length != 1) {
      System.out.println("Usage: KernelCount <input file>.");
      System.exit(1);
      }

    infs = new FileInputStream(argv[0]);             
    ins = new StreamTokenizer(new InputStreamReader(infs));

    ins.lowerCaseMode(false);
    ins.eolIsSignificant(false);
    ins.parseNumbers();
    
    matrix = new Vector[MAXSIZE];
    columnid = new long[MAXSIZE][2];
    mxsize = 0;
    i = -1;
    j = 0;
    column = new Vector(100,100);
    
    System.out.print("Reading matrix...  ");
    while (true) {
      column.removeAllElements();
      ans  = readLine(ins, column);
      if (ans == COLUMN) {
        if (mxsize >= MAXSIZE) {
          System.out.println("Hey! This matrix has more than " + 
                           MAXSIZE +" columns.  Give me a break...");
          System.exit(1);
          } 
        matrix[mxsize] = (Vector) column.clone();
        LinAlgUtil.reduce(matrix[mxsize]);
        columnid[mxsize][0] = i;
        columnid[mxsize][1] = j++;
        mxsize++;
        }
      else if (ans == EOF)
        break;
      else {	// found a "Graph x" line
        i = ans; 
        j = 0;
        }
      }
    System.out.println("Read "+ mxsize +" columns.");

    System.out.print("Eliminating ... ");
    kerneldim = imagedim = 0;         
    for (j=0; j<mxsize; j++) {        		// compute the kernel

        if (matrix[j].size() == 0) {		// zero column in matrix:
          									// found a kernel element            
          kerneldim++;
          System.out.print("*");
          matrix[j] = null;
          continue;
          }
       			// non-zero column in matrix: a basis vector of the image
        imagedim++;
       
        mxelt = (long[])matrix[j].elementAt(0); 
        System.out.print("("+ j +","+ mxelt[0]);
        largestCoeff = 0;

        for (k=j+1; k<mxsize; k++) {				// check each subsequent
          boolean occurs = false;					// column if the main 
          for (p=0; p<matrix[k].size(); p++) {		// coeff of matrix[j]
            oldelt = (long[])(matrix[k].elementAt(p));	// occurs in it
            if (mxelt[1] == oldelt[1] && mxelt[2] == oldelt[2]) {
              occurs = true; 
              break;
              }
           }
         if (occurs) {
            mult = mxelt[0];
            subt = oldelt[0];
            q = LinAlgUtil.gcd(mult, subt);
            mult = mult/q;
            subt = subt/q;
            if (subt > largestCoeff)
              largestCoeff = subt;
            if (subt < -largestCoeff)						// this it the
              largestCoeff = -subt;							// Gaussian elim.
            LinAlgUtil.multiply(matrix[k], mult);			// part gufe
            LinAlgUtil.addColumn(matrix[j], -subt, matrix[k]);
            LinAlgUtil.reduce(matrix[k]);
            }
          }
        matrix[j] = null;
        System.out.print(","+ largestCoeff +")");
        }
     
    infs.close();

    System.out.println();
    System.out.println("kernel dim = "+ kerneldim +"; image dim = "+
          imagedim +"  "+ new java.util.Date());
    }
    
   
  final static public int COLUMN = -2;
  final static public int EOF = -1;	// non-neg values mean "Graph X" lines
  
  public static int readLine(StreamTokenizer s, Vector bag)
                                                      throws IOException {
    long mxelt[] = new long[3];
       
    s.nextToken();
    if (s.ttype == StreamTokenizer.TT_EOF)
      return EOF;
      
    while (s.ttype == StreamTokenizer.TT_NUMBER) {
      mxelt[0] = (long)s.nval;
      s.nextToken();
      mxelt[1] = (long)s.nval;
      s.nextToken();
      mxelt[2] = (long)s.nval;
      s.nextToken();
      bag.addElement(mxelt.clone());
      }
    
    if (s.ttype == (int)';')
      return COLUMN;
    
    if (s.ttype == StreamTokenizer.TT_WORD && s.sval.equals("Graph")) {
      s.nextToken();
      return (int)s.nval;
      }
    
    System.out.println("Error reading "+s);
    System.exit(1);
    return EOF;		// well, it doesn't, but the compiler requires it
    }      
    
}
