/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* ListGraphs.java                                           */
/* constructs graphs of the same rank, with one less vertex  */
/* by contracting edges                                      */

package OutH;

import java.util.Vector;
import java.io.*;

public class ListGraphs 
{
  static public void main(String[] args) throws IOException
	{
    int i, j, k, n;
    boolean isNew;
    Graph oldGraph, newGraph;
    Vector oldBag, newBag;
    
    if (args.length != 2) {
      System.out.println("Usage: ListGraphs inputgraphs "+
                               "outputgraphs");
      System.exit(1);
      }
    
    oldBag = new Vector(10,10);
    
    FileInputStream infs = new FileInputStream(args[0]);
    ObjectInputStream ins = new ObjectInputStream(infs);
    try {
      while (true) {
        oldGraph = (Graph)ins.readObject();
        oldGraph.treeBag = null;
        oldGraph.symBag = null;
        oldBag.addElement(oldGraph);
        }
      }
    catch(EOFException e) { }
    catch (ClassNotFoundException e) {
      System.out.println("Class not found.");
      System.exit(1);
      }
    infs.close();

    System.out.print(oldBag.size() + " graphs read.\r\n"+
                      "Processing graphs:");
  
    newBag = new Vector(100,100);
   
    for (n=0; n<oldBag.size(); n++) {
      oldGraph = (Graph)oldBag.elementAt(n);
      System.out.print(" " + n);
    
      for (i=0; i<oldGraph.size; i++)
        for (j=i+1; j<oldGraph.size; j++)
          if (oldGraph.m[i][j] != 0)
            {
            newGraph = new Graph(oldGraph);      
            newGraph.contractEdge(i, j);
            newGraph.toNormalForm();	// finds normal form
            isNew = true;				// and automorphism group
        
            for (k=0; k<newBag.size(); k++)
              if (newGraph.equals((Graph)newBag.elementAt(k))) {
                isNew = false;
                break;
                }						
            if (isNew) {			// not in the bag yet 
              newBag.addElement(newGraph);
              System.out.print("*");
              }
            else
              System.out.print(".");
            }
      }	// end of "for n" loop

    FileOutputStream outfs = new FileOutputStream(args[1]);
    ObjectOutputStream outs = new ObjectOutputStream(outfs);
    for (n=0; n<newBag.size(); n++)
      outs.writeObject((Graph)newBag.elementAt(n));
    outs.close();
    outfs.close();

    System.out.println("\r\nWritten " + n + " graphs at " +
                       new java.util.Date());
    }
}
