/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Find the subtrees of the graphs,                         */
/* i.e., cubes of the complex of a given rank and dimension */
/* (rank = rank of the parent graph,                        */
/*  dimension = # of edges of the subtree)                  */

package OutH;

import java.util.Vector;
import java.io.*;

public class ListTrees
{
  public static void main(String argv[]) throws IOException
  {
    int i, j, k, n,
        number, tn, min, max, sum,
        rank, dim;
    Graph graph = null;
    Tree t, t1;
    int[] subset;
    String infile, outfile;
    
    FileInputStream infs;
    ObjectInputStream ins;
    FileOutputStream outfs;
    ObjectOutputStream outs;

    if (argv.length < 2) {
      System.out.println("Usage: ListTrees <rank> <dimension>.");
      System.exit(1);
      }
      
    rank = Float.valueOf(argv[0]).intValue();
    dim = Float.valueOf(argv[1]).intValue();

    outfile = "graphs"+ rank +"_"+ dim +".out";
    outfs = new FileOutputStream(outfile, false);
    outs = new ObjectOutputStream(outfs);

    min = 10000;
    max = 0;
    sum = 0;
    number = 0;
    
  for (n=dim; n<=2*rank-3; n++)
    {
    infile = "gr"+ rank +"_"+ n +".out";
    infs = new FileInputStream(infile);
    ins = new ObjectInputStream(infs);

    while (true) {
      try { graph = (Graph)ins.readObject(); }
      catch (EOFException e) {
        break;
        }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
         
      number++;
      subset = new int[graph.noEdges];
      graph.treeBag.removeAllElements();
      System.gc();
       
      for (k=0; k<dim; k++)		// initialize subset to first
        subset[k] = 1;			// 'dim' edges
          
      do {
        t = subTree(graph, subset);
        if (t.isAForest() && !t.hasOddSymmetry()) {
          t.toNormalForm();
          t.coeff = 1;
          addTree(graph.treeBag, t);
          }
      } while (nextSubset(subset));

      outs.writeObject(graph);
        
      tn = graph.treeBag.size();
      if (tn<min)
        min = tn;
      if (tn>max)
        max = tn;
      sum += tn;
      System.out.print("("+ number +","+ tn +") ");
      }
    ins.close();   infs.close();
    }

    outs.close();  outfs.close();    
    System.out.println("done. "+new java.util.Date()+"\nmin: "+min+
            ", max: "+max+", sum: "+sum+" (out of "+number+")");
    }
    
    
  public static Tree subTree(Graph g, int[] subset) {
    int i, j, n, count;
    Tree t = new Tree(g.size);
    
    n = count = 0;
    for (i=0; i<g.size; i++)
      for (j=i+1; j<g.size; j++)
        if (g.m[i][j] != 0) {
          if (subset[n] != 0) {
            t.m[i][j] = (byte)++count;
            }
          n++;
          }
    t.allEdges = count;
    t.parent = g;
    t.coeff = 1;
    return t;
    }
    

  private static boolean nextSubset(int[] subset) {
    int i, j, k;
    boolean isLast = true;
    
    k = -1;
    for (i=0; i<subset.length-1; i++) {
      if (subset[i] == 1) {				// find first occurrence
        k++;							// of a 1 followed by a 0
        if (subset[i+1] == 0) {
          isLast = false;
          break;
          }
        }
      }
    if (isLast)
      return false;
    
    for (j=0; j<k; j++)		// advance to next subset (of same
      subset[j] = 1;		// number of elements) in reverse-
    for (; j<=i; j++)		// lexicographic order
      subset[j] = 0;
    subset[j] = 1;

    return true;
    }
    
    
  private static void addTree(Vector bag, Tree tree) {
    int i;
    Tree t1;
    
    for (i=0; i<bag.size(); i++) {		// check if the tree is already
      t1 = (Tree)bag.elementAt(i);		// in the bag
      if (tree.equals(t1))
        return;
      }
        
    bag.addElement(tree);
    }
}
