/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Writes the boundary vectors as a linear combination of the */
/* kernel vectors.                                            */
/*                                                            */

package OutH;
import java.util.*;
import java.io.*;

public class Quotient
{
  public static void main(String argv[]) {
    int i, j, k;
    Vector kernelBag, boundaryBag, column, col2;
    String kernelFile, boundaryFile, outputFile = "";
    long[] mxelt, oldelt = null;
    long mult, subt, p;
    float coeff;
    boolean occurs;
    
    if (argv.length == 3) {
      kernelFile = argv[0];
      boundaryFile = argv[1];
      outputFile = argv[2];
      }
    else {
      System.out.println("Usage: java OutH.Quotient kernelx imagex+1 output");
      System.exit(1);
      kernelFile = boundaryFile = null;
      }
    
    try {      
      FileInputStream kerfs = new FileInputStream(kernelFile);             
      StreamTokenizer kers = new StreamTokenizer(new InputStreamReader(kerfs));
      FileInputStream bdyfs = new FileInputStream(boundaryFile);             
      StreamTokenizer bdys = new StreamTokenizer(new InputStreamReader(bdyfs));
      kers.lowerCaseMode(false);	bdys.lowerCaseMode(false);
      kers.eolIsSignificant(false);	bdys.eolIsSignificant(false);
      kers.parseNumbers();			bdys.parseNumbers();

      kernelBag = new Vector(20,20);
      column = new Vector(100,100);    
      while (Kernel.readLine(kers, column) == Kernel.COLUMN) {
        kernelBag.addElement(column.clone());
        column.removeAllElements();    
        }
      boundaryBag = new Vector(20,20);
      column.removeAllElements();    
      while (Kernel.readLine(bdys, column) == Kernel.COLUMN) {
        boundaryBag.addElement(column.clone());
        column.removeAllElements();    
        }
      kerfs.close();
      bdyfs.close();

      System.out.print("eliminating... ");
      for (i=0; i<kernelBag.size(); i++) {
        column = (Vector)kernelBag.elementAt(i);
        mxelt = (long[])column.elementAt(0);
        for (j=i+1; j<kernelBag.size(); j++) {		// check each subsequent
          col2 = (Vector)kernelBag.elementAt(j);	// column if the main 
          occurs = false;							// coeff occurs in it				
          for (k=0; k<col2.size(); k++) {
            oldelt = (long[])col2.elementAt(k);
            if (mxelt[1] == oldelt[1] && mxelt[2] == oldelt[2]) {
              occurs = true; 
              break;
              }
            }
          if (occurs) {
            mult = mxelt[0];
            subt = oldelt[0];
            p = LinAlgUtil.gcd(mult, subt);
            mult = mult/p;
            subt = subt/p;
            LinAlgUtil.multiply(col2, mult);
            LinAlgUtil.addColumn(column, -subt, col2);
            }
          }
        }
        
      System.out.print("writing file... ");
      FileOutputStream outfs = new FileOutputStream(outputFile, false);             
      PrintWriter outs = new PrintWriter(outfs);

      for (i=0; i<boundaryBag.size(); i++) {
        column = (Vector)boundaryBag.elementAt(i);
        for (j=0; j<kernelBag.size(); j++) {
          col2 = (Vector)kernelBag.elementAt(j);      
          mxelt = (long[])col2.elementAt(0);
          occurs = false;			
          for (k=0; k<column.size(); k++) {
            oldelt = (long[])column.elementAt(k);
            if (mxelt[1] == oldelt[1] && mxelt[2] == oldelt[2]) {
              occurs = true; 
              break;
              }
            }
          if (occurs) {
            mult = mxelt[0];
            subt = oldelt[0];
            p = LinAlgUtil.gcd(mult, subt);
            mult = mult/p;
            subt = subt/p;
            LinAlgUtil.multiply(column, mult);
            LinAlgUtil.addColumn(col2, -subt, column);
            coeff = (float)subt / (float)mult;
            outs.print(coeff +"x_"+ j +" + ");
            }
          }
        outs.println(column.size());
        }
      
      outs.close(); outfs.close();
      }
      catch (IOException e) {
        System.out.println("I/O error.");
        System.exit(1);
        }
      }
}
