/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Class for storing a tree on n numbered vertices. */
/*                                                  */

package OutH;

public class Tree extends SimpleGraph implements java.io.Serializable
{
  
  public int coeff;
  public Graph parent;
  
  public Tree(int n) {
    super(n);
    noEdges = -1;				// all trees have noEdges == -1
    coeff = 1;					// actual # of edges is in allEdges
    }
 

  public Tree(Graph g) {		// lecsupasztjuk
    this(g.size);	
    int i, j, n;
    
    n = 1;
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)
        if (g.m[i][j] != 0)
          m[i][j] = (byte)n++;

    allEdges = g.noEdges;
    parent = g;
    }


  public Tree(Tree t) {
    this(t.size);
    int i, j;

    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)
        m[i][j] = t.m[i][j];

    allEdges = t.allEdges;
    parent = t.parent;
    coeff = t.coeff;
    }


  public boolean isConnected() {
    int i, j, k;
    int tmp[][];
    
    tmp = new int[size][size];
        
    for (i=0; i<size; i++)
      for (j=i; j<size; j++)
        if (m[i][j] != 0)
          tmp[i][j] = 1;
 
    for (i=0; i<size-1; i++) {		// 1. check whether vertex 1 is connected
      for (j=i+1; j<size; j++)		//      to anything; NO -> return false
        if (tmp[i][j] != 0)			// 2. if 1 is conn to j, delete vertex 1  
          break;					//      and conn all neighbors of 1 to j
      if (j == size)				//      by new edges;
        return false;				//      new graph is conn iff old one is
      for (k=j+1; k<size; k++)		// 3. continue with vertex 2 etc.
        tmp[j][k] |= tmp[i][k];
      }
    return true;
    }        


  public boolean isAForest() {
    int i, j, k;
    int tmp[][];
    
    tmp = new int[size][size];
        
    for (i=0; i<size; i++) {
      if (m[i][i] != 0)				// there is a loop
        return false;
      for (j=i+1; j<size; j++)
        if (m[i][j] != 0)
          tmp[i][j] = 1;
      }
 
    for (i=0; i<size; i++) {
      for (j=i+1; j<size; j++)
        if (tmp[i][j] != 0)
          break;
      if (j<size && tmp[i][j] > 1)	// there is a multiple edge
        return false;
      for (k=j+1; k<size; k++)
        tmp[j][k] += tmp[i][k];
      }
    return true;
    }        


  public int[] kthEdge(int k) {		// find kth edge of the tree
    int i, j, r[] = new int[2];
        
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)
        if (m[i][j] == k+1) {
          r[0] = i;
          r[1] = j;
          return r;
          }
    r[0] = r[1] = -1;				// error: there is no kth edge
    return r;
    }
    
    
  public void removeEdge(int k) {
    int r[], i, j, n;				
    
    r = kthEdge(k);
    n = m[r[0]][r[1]];		// remove edge and remember its label
    m[r[0]][r[1]] = 0;		//  (should be k+1, but who knows...)
    
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)		// decrease all higher labels
        if (m[i][j] > n)
          m[i][j]--;
    
    allEdges--;
    }     
    
    
  public void contractEdge(int k) {		// this is the only operation
    int[] r	= kthEdge(k);				//  which changes the parent
 
    parent.contractEdge(r[0], r[1]);	// parent is unnormalized
    super.contractEdge(r[0], r[1]);
    
    int i, j;
    
    m[r[0]][r[0]] = 0;				// remove loop; this loop is an
    								// artifact of numbering edges
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)
        if (m[i][j] > k+1)			// decrease index of later edges
          m[i][j]--;
    }
    

  public void contractLean(int k) {	// this version is used when the
    int[] r	= kthEdge(k);			//  parent is contracted already

    super.contractEdge(r[0], r[1]);

    int i, j;
    
    m[r[0]][r[0]] = 0;

    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)
        if (m[i][j] > k+1)
          m[i][j]--;
    }


  public Permutation orderOfEdges() {
    int i, j, n;
    Permutation sigma = new Permutation(allEdges);
    
    n = 0;
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)
        if (m[i][j] != 0)
          sigma.p[n++] = m[i][j] - 1;
    
    return sigma;
    }


  public void toNormalForm() {
    int i, j, p, n;					// put tree in normal form;
    Tree t, tmax;					// we are assuming the parent is
    Permutation sigma, smax;		// in normal form already
				
	coeff *= orderOfEdges().sign();
				
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)		// forget the numbering of
        if (m[i][j] != 0)			//  the edges
          m[i][j] = 1;

    tmax = new Tree(this);
    smax = new Permutation(size);

    for (p=0; p<parent.symBag.size(); p++) {
      t = new Tree(this);
      sigma = (Permutation)parent.symBag.elementAt(p);
      t.permute(sigma);
      if (SimpleGraph.order(t, tmax) == 1) {
        tmax = t;
        smax = sigma;
        }
      }

    n = 1;
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)		// number the edges again
        if (m[i][j] != 0)
          m[i][j] = (byte)n++;
    permute(smax);
    coeff *= orderOfEdges().sign();
    n = 1;
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)		// and finally put them in
        if (m[i][j] != 0)			//  increasing order
          m[i][j] = (byte)n++;
    }


  public boolean hasOddSymmetry() {
    int i, j, p, mysign;
    Permutation sigma;
    Tree t, tnn;
    
    tnn = new Tree(this.size);
    for (i=0; i<size; i++)
      for (j=i+1; j<size; j++)	// make an unnumbered copy of this
        if (m[i][j] != 0)
          tnn.m[i][j] = 1;
    tnn.allEdges = allEdges;
    
    mysign = orderOfEdges().sign();
    
    for (p=0; p<parent.symBag.size(); p++) {
      t = new Tree(tnn);
      sigma = (Permutation)parent.symBag.elementAt(p);
      t.permute(sigma);
      if (tnn.equals(t)) {
        Tree t2 = new Tree(this);
        t2.permute(sigma);
        if (mysign != t2.orderOfEdges().sign())
          return true;
        }
      }
      
    return false;
    }
    
    
  public String toString() {
    String str = super.toString() +"; coeff: "+ coeff +"\r\n";   
    return str;
    }  

/*  
  public static void main(String args[]) {		// testing...
    byte sample[][] = {{0, 2, 0, 1, 0 },
                       {0, 0, 0, 0, 1 },
                       {0, 0, 0, 1, 1 },
                       {0, 0, 0, 0, 1 },
                       {0, 0, 0, 0, 0 }};
    Graph g = new Graph(5);
    
    for (int i=0; i<g.size; i++)
      for (int j=i; j<g.size; j++)
        g.m[i][j] = sample[i][j];
    g.noEdges = 6;
    g.allEdges = 7;
    g.noLoops = 0;
    g.noMults = 2;
    
    g.toNormalForm();    
    System.out.println(g);
    
    Tree t = new Tree(g);
    t.toNormalForm();
    System.out.println(t);
    
    for (int i=0; i<t.noEdges; i++) {
      System.out.println("Contracting edge #"+ i);
      Graph gg = new Graph(g);
      Tree tt = new Tree(gg);
      tt.contractEdge(i);
      System.out.println(tt);
      }
    }
*/    
}
