/*                                      */
/* Out(F_n) computation                 */
/* Ferenc Gerlits, PhD thesis, Aug 2002 */
/*                                      */
/* Trivalent.java                                             */
/* lists all trivalent graphs of rank n (i.e., 2n-2 vertices) */
/* using the list of trivalent graphs of rank n-1             */
/*  e.g., trivalent4.out -> trivalent5.out                    */
/*                                                            */

package OutH;

import java.util.Vector;
import java.io.*;

public class Trivalent 
{
  static public void main(String[] args) throws IOException
	{
    int i, j, k, n;
    boolean isNew;
    Graph oldGraph, newGraph;
    Vector oldBag, newBag;
    
    if (args.length != 2) {
      System.out.println("Usage: ListTrivalent inputgraphs "+
                               "outputgraphs");
      System.exit(1);
      }
    
    oldBag = new Vector(10,10);
    
    if (args[0].equals("null")) {		// generating rank 2 graph
      Graph basicGraph = new Graph(2);
      basicGraph.m[0][1] = 3;
      basicGraph.noEdges = 1;
      basicGraph.allEdges = 3;
      basicGraph.noMults = 2;
      oldBag.addElement(basicGraph);
      }
    else {
      FileInputStream fs = null;
      ObjectInputStream s;
      fs = new FileInputStream(args[0]);
      s = new ObjectInputStream(fs);
      try {
        while (true) {
          oldGraph = (Graph)s.readObject();
          oldGraph.treeBag = null;
          oldGraph.symBag = null;
          oldBag.addElement(oldGraph);
          }
        }
      catch(EOFException e) { }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
      fs.close();
      }
    System.out.print(oldBag.size() + " graphs read.\r\n"+
                      "Processing graphs:");
  
    newBag = new Vector(100,100);
   
    for (n=0; n<oldBag.size(); n++) {
      oldGraph = (Graph)oldBag.elementAt(n);
      System.out.print(" " + n);
    
      for (i=0; i<oldGraph.allEdges; i++)
        for (j=i; j<oldGraph.allEdges; j++) {
          newGraph = oldGraph.addEdge(i, j);
          newGraph.toNormalForm();	// finds normal form
          isNew = true;				// and automorphism group
        
          for (k=0; k<newBag.size(); k++)
            if (newGraph.equals((Graph)newBag.elementAt(k))) {
              isNew = false;
              break;
              }						
          if (isNew) {			// not in the bag yet 
            newBag.addElement(newGraph);
            System.out.print("*");
            }
          else
            System.out.print(".");
          }
      }	// end of "for n" loop

    FileOutputStream fs = new FileOutputStream(args[1]);
    ObjectOutputStream s = new ObjectOutputStream(fs);
    for (n=0; n<newBag.size(); n++)
      s.writeObject((Graph)newBag.elementAt(n));
    s.close();
    fs.close();

    System.out.println("\r\nWritten " + n + " graphs at " +
                       new java.util.Date());
    }
}
